==> chart-0.2.2.tar.gz: lib/src/line.dart <==
part of chart;

/**
 * Code based on charts.js by Nick Downie, http://chartjs.org/
 * 
 * (Partial) Dart implementation done by Symcon GmbH, Tim Rasim
 * 
 */

class Line extends DartChart {
  
  int maxSize;
  int scaleHop;
  Map calculatedScale;
  int labelHeight;
  int scaleHeight;
  Map<String, Object> valueBounds;
  int labelHop; //x-distance between two x-axis labels
  int widestXLabel;
  double xAxisLength;
  double yAxisPosX;
  double xAxisPosY;
  int rotateLabels = 0;


  Line(data, config):super(data, config) {

    defaultConfiguration({
        'scaleOverlay' : false, 
        'scaleOverride' : false, 
        'scaleSteps' : null, 
        'scaleStepWidth' : null, 
        'scaleStartValue' : null, 
        'scaleLineColor' : "rgba(0,0,0,.1)", 
        'scaleLineWidth' : 1, 
        'scaleShowLabels' : true, 
        'scaleLabel' : new Map<double, String>(), 
        'scaleFontFamily' : "'Verdana'", 
        'scaleFontSize' : 12, 
        'scaleFontStyle' : "normal", 
        'scaleFontColor' : "#666", 
        'scaleShowGridLines' : true, 
        'scaleGridLineColor' : "rgba(0,0,0,.05)", 
        'scaleGridLineWidth' : 1, 
        'bezierCurve' : true, 
        'pointDot' : true, 
        'pointDotRadius' : 4.0, 
        'pointDotStrokeWidth' : 2, 
        'datasetStroke' : true, 
        'datasetStrokeWidth' : 2, 
        'datasetFill' : true, 
        'animation' : true, 
        'animationSteps' : 60, 
        'animationEasing' : "easeOutQuart", 
        'onAnimationComplete' : null
    });
    
  }
  
  void show(Element parentNode){
    
    super.show(parentNode);

    Map<String, int> drawingSizes = calculateDrawingSizes(this.maxSize, this.rotateLabels);
    this.labelHeight = drawingSizes['labelHeight'];
    this.scaleHeight = drawingSizes['scaleHeight'];
    this.widestXLabel = drawingSizes['widestXLabel']; 
    
    valueBounds = getValueBounds(this.scaleHeight, this.labelHeight);
    if (!getConfiguration('scaleOverride')) {
      this.calculatedScale = calculateScale(this.scaleHeight, this.valueBounds['maxSteps'], this.valueBounds['minSteps'], this.valueBounds['maxValue'], this.valueBounds['minValue']);
    } else {
      this.calculatedScale = {
          'steps' : getConfiguration('scaleSteps'), 'stepValue' : getConfiguration('scaleStepWidth'), 'graphMin' : getConfiguration('scaleStartValue'), 'labels' : []
      };
      populateLabels(this.calculatedScale['labels'], this.calculatedScale['steps'], getConfiguration('scaleStartValue'), getConfiguration('scaleStepWidth'));
    }

    this.scaleHop = (this.scaleHeight / (this.calculatedScale['steps'])).floor();
    calculateXAxisSize();
    
    animationLoop();    
    
  }

  calculateXAxisSize() {
    
    double longestText = 1.0;
    //if we are showing the labels
    if (getConfiguration('scaleShowLabels')) {
      this.context.font = getConfiguration('scaleFontStyle').toString() + " " + getConfiguration('scaleFontSize').toString() + "px " + getConfiguration('scaleFontFamily').toString();
      for (int i = 0; i < this.data['labels'].length; i++) {
        double measuredText = this.context.measureText(this.data['labels'][i].toString()).width;
        longestText = (measuredText > longestText) ? measuredText : longestText;
      }
      
      //also calculate the width of the scales, so they won't be cut off.
      for (int i = 0; i < calculatedScale['steps']; i++) {
        double measuredText = this.context.measureText(calculatedScale['labels'][i]).width;
        longestText = (measuredText > longestText) ? measuredText : longestText;
      }
      
      //Add a little extra padding from the y axis
      longestText = longestText + 10;
    }

    this.xAxisLength = this.width.toDouble() - longestText - this.widestXLabel.toDouble();
    
    //this.xAxisLength / (this.data['labels'].length -1) would be correct, but we want one step more
    this.labelHop = (this.xAxisLength / (this.data['labels'].length)).floor();
    
    this.yAxisPosX = this.width.toDouble() - this.widestXLabel / 2 - this.xAxisLength;
    
    this.xAxisPosY = this.scaleHeight.toDouble() + getConfiguration('scaleFontSize')/2;
    
  }
  
  
  /**
   * 
   * draw one step which can consists of two points or multiple inter steps
   * 
   * pointBefore: the last point before this step
   * 
   * returns: the last point inside this step
   */
  Object drawStep(dynamic previousValue, dynamic currentValue, Map dataset, num lastStartX, double currentX, double stepSizeX, double lastStepSizeX, double animPc){  
    
    if (previousValue == null && currentValue == null){
      //if the point before the current point was null, only move the pencil to the point
      this.context.moveTo(xPos(lastStartX), yPos(0.0, animPc));      
      lastStartX = currentX;
      return {
        'currentValue' : null,
        'lastStartX' : lastStartX,
        'lastStepSizeX' : stepSizeX
      };
    }
    
    if (previousValue != null && currentValue == null){
      // the last set of points ends with the last point
      closePath(dataset,lastStartX, currentX -lastStepSizeX);      
      
      return {
        'currentValue' : null,
        'lastStartX' : lastStartX,
        'lastStepSizeX' : stepSizeX
      };
    }
    
    if (previousValue != null && (currentValue is List && currentValue.last == null)){
      //the first value in the list is empty
      
      Object data = {
                     'currentValue' : previousValue,
                     'lastStartX' : lastStartX,
                     'lastStepSizeX' : lastStepSizeX
      };
      stepSizeX = 1 / currentValue.length;
      for (int z = 0; z < currentValue.length; z++)
        data = drawStep(data['currentValue'], currentValue[z],dataset, data['lastStartX'], currentX + stepSizeX * z, stepSizeX, data['lastStepSizeX'], animPc);  

      
      return {
        'currentValue' : data['currentValue'],
        'lastStartX' : data['lastStartX'],
        'lastStepSizeX' : stepSizeX
      };
    }

    if (currentValue != null){
      //continue context from before
      if (previousValue == null){
        lastStartX = currentX;
        if (currentValue is List && currentValue.first is num){
          this.context.moveTo(xPos(lastStartX), yPos(currentValue.first, animPc));
        } else if (currentValue is num){
          this.context.moveTo(xPos(lastStartX), yPos(currentValue, animPc));
        }
      } else if (currentValue is num){
        if (getConfiguration('bezierCurve')) {
          if (previousValue == null){
            previousValue = 0;
          }

          this.context.bezierCurveTo(xPos(currentX  - math.min(stepSizeX, lastStepSizeX)/2), yPos(previousValue.toDouble(), animPc), xPos(currentX  - math.min(stepSizeX, lastStepSizeX)/2), yPos(currentValue.toDouble(), animPc), xPos(currentX), yPos(currentValue, animPc));
        } else {
          this.context.lineTo(xPos(currentX), yPos(currentValue, animPc));
        }
      } 
      
      if (currentValue is List){
        Object data = {
                       'currentValue' : previousValue,
                       'lastStartX' : lastStartX,
                       'lastStepSizeX' : lastStepSizeX
        };
        stepSizeX = 1 / currentValue.length;
        if (previousValue == null){
          data['currentValue'] = currentValue.first;
          data['lastStartX'] = currentX + stepSizeX;
          for (int z = 1; z < currentValue.length; z++){
            data = drawStep(data['currentValue'], currentValue[z],dataset, data['lastStartX'], currentX + stepSizeX * z, stepSizeX, data['lastStepSizeX'], animPc);
          }
        } else {
          for (int z = 0; z < currentValue.length; z++){
            data = drawStep(data['currentValue'], currentValue[z],dataset, data['lastStartX'], currentX + stepSizeX * z, stepSizeX, data['lastStepSizeX'], animPc);  
          }
        }
      lastStartX = data['lastStartX'];
      currentValue = data['currentValue'];
      }
      
      return {
        'currentValue' : currentValue,
        'lastStartX' : lastStartX,
        'lastStepSizeX' : stepSizeX
      };
    }     
    
  }
  
  void drawLines(double animPc) {
    
    //for all: drawStep
    //then draw end if last is not null
    
    this.data['datasets'].forEach( (Map dataset){
      this.context.strokeStyle = dataset['strokeColor'];
      this.context.lineWidth = getConfiguration('datasetStrokeWidth');
      
      this.context.beginPath();
      
      if (dataset['data'][0] is List && dataset['data'][0][0] != null ){
        this.context.moveTo(this.yAxisPosX, this.xAxisPosY - animPc * (calculateOffset(dataset['data'][0][0].toDouble(), this.calculatedScale, this.scaleHop)));
      } else if (dataset['data'][0] is num){
        this.context.moveTo(this.yAxisPosX, this.xAxisPosY - animPc * (calculateOffset(dataset['data'][0].toDouble(), this.calculatedScale, this.scaleHop)));
      }
      
      List test = new List();
      int x = 0;
      Object data = {
       'currentValue' : null,
       'lastStartX' : 0,
       'lastStepSizeX' : 1.0
      };
      dataset['data'].forEach( (value){
        data = drawStep(data['currentValue'], value, dataset, data['lastStartX'], x.toDouble(), 1.0, data['lastStepSizeX'], animPc);
        x++;
      });
      
      if ((dataset['data'].last is num) || (dataset['data'].last is List && dataset['data'].last.last != null)){ 
        closePath(dataset, data['lastStartX'], dataset['data'].length -1);
      } 
      
      if (getConfiguration('pointDot')) {
        this.context.fillStyle = dataset['pointColor'];
        this.context.strokeStyle = dataset['pointStrokeColor'];
        this.context.lineWidth = getConfiguration('pointDotStrokeWidth');
        for (var k = 0; k < dataset['data'].length; k++) {
          if (dataset['data'][k] is List){
            //multiple intermediary steps   
            double stepSizeX = 1 / dataset['data'][k].length;
            int n = 0;
            dataset['data'][k].forEach( (item) {
              if (item != null && item is num){
                this.context.beginPath();
                this.context.arc(xPos(k + stepSizeX *n++ ), this.xAxisPosY - animPc * (calculateOffset(item.toDouble(), this.calculatedScale, this.scaleHop)), getConfiguration('pointDotRadius'), 0, math.PI * 2, true);
                this.context.fill();
                this.context.stroke();
              }
            });
           } else if (dataset['data'][k] is num) {
            //one step
            this.context.beginPath();
            this.context.arc(xPos(k), this.xAxisPosY - animPc * (calculateOffset(dataset['data'][k].toDouble(), this.calculatedScale, this.scaleHop)), getConfiguration('pointDotRadius'), 0, math.PI * 2, true);
            this.context.fill();
            this.context.stroke();
           }
        }
      }
    });
    
  }
  
  void closePath(Map dataset, num startX, num endX){

    this.context.stroke();
       
    if (getConfiguration('datasetFill')) {
      if (dataset['data'][endX.floor()] is List && dataset['data'][endX.floor()].last != null && endX % 1 == 0){
        //1 - 1 / dataset['data'].last.length means the last step exclusive a next full step (confusing but better than no comment at all)
        drawDatasetFill(dataset, startX, endX + 1 - 1 / dataset['data'][endX.floor()].length);
      } else {
        drawDatasetFill(dataset, startX, endX);
      }
    } else {
      this.context.closePath();
      this.context.beginPath();
    }  

  }
  
  /*
   * Draws the background between the points startX and endX
   */
  void drawDatasetFill(Map dataset, num startX, num endX){
      
      this.context.strokeStyle = dataset['fillColor'];
      this.context.lineTo(this.yAxisPosX + (this.labelHop * endX), this.xAxisPosY);
      this.context.stroke();
    
      this.context.strokeStyle = "rgba(0,0,0,0)"; //transparent
      this.context.lineTo(this.yAxisPosX + (this.labelHop * startX), this.xAxisPosY);
      this.context.stroke();
    
      this.context.closePath();
      this.context.fillStyle = dataset['fillColor'];
      this.context.fill();
      
      this.context.beginPath();
      this.context.strokeStyle = dataset['strokeColor'];

  }

  double yPos(num value, double animPc) {
    
    return this.xAxisPosY - animPc * (calculateOffset(value.toDouble(), this.calculatedScale, this.scaleHop));
  
  }

  //FIXME: iterations name is confusing. (suggests it is of the type int, but istn't)
  double xPos(num iteration) {
    
    return this.yAxisPosX + (this.labelHop * iteration);
    
  }

  //@override
  void drawScale() {
    
    //X axis line
    this.context.lineWidth = getConfiguration('scaleLineWidth');
    this.context.strokeStyle = getConfiguration('scaleLineColor');
    this.context.beginPath();
    this.context.moveTo(this.width - this.widestXLabel / 2 + 5, this.xAxisPosY);
    this.context.lineTo(this.width - (this.widestXLabel / 2) - this.xAxisLength - 5, this.xAxisPosY);
    this.context.stroke();


    if (this.rotateLabels > 0) {
      this.context.save();
      this.context.textAlign = 'right';
    } else {
      this.context.textAlign = 'center';
    }
    this.context.fillStyle = getConfiguration('scaleFontColor');
    for (var i = 0; i < this.data['labels'].length; i++) {
      this.context.save();
      if (this.rotateLabels > 0) {
        this.context.translate(this.yAxisPosX + i * this.labelHop, this.xAxisPosY + getConfiguration('scaleFontSize'));
        this.context.rotate(-(this.rotateLabels * (math.PI / 180)));
        this.context.fillText(this.data['labels'][i].toString(), 0, 0);
        this.context.restore();
      } else {
        this.context.fillText(this.data['labels'][i].toString(), this.yAxisPosX + i * this.labelHop, this.xAxisPosY + getConfiguration('scaleFontSize') + 3);
      }

      this.context.beginPath();
      this.context.moveTo(this.yAxisPosX + i * this.labelHop, this.xAxisPosY + 3);

      //Check i isnt 0, so we dont go over the Y axis twice.
      if (getConfiguration('scaleShowGridLines') && i > 0) {
        this.context.lineWidth = getConfiguration('scaleGridLineWidth');
        this.context.strokeStyle = getConfiguration('scaleGridLineColor');
        this.context.lineTo(this.yAxisPosX + i * this.labelHop, 5);
      } else {
        this.context.lineTo(this.yAxisPosX + i * this.labelHop, this.xAxisPosY + 3);
      }
      this.context.stroke();
    }

    //Y axis
    this.context.lineWidth = getConfiguration('scaleLineWidth');
    this.context.strokeStyle = getConfiguration('scaleLineColor');
    this.context.beginPath();
    this.context.moveTo(this.yAxisPosX, this.xAxisPosY + 5);
    this.context.lineTo(this.yAxisPosX, 5);
    this.context.stroke();

    this.context.textAlign = "right";
    this.context.textBaseline = "middle";
    for (int j = 0; j <= calculatedScale['steps']; j++) {
      this.context.beginPath();
      this.context.moveTo(this.yAxisPosX - 3, this.xAxisPosY - ((j + 1) * this.scaleHop));
      if (getConfiguration('scaleShowGridLines')) {
        this.context.lineWidth = getConfiguration('scaleGridLineWidth');
        this.context.strokeStyle = getConfiguration('scaleGridLineColor');
        this.context.lineTo(this.yAxisPosX + this.xAxisLength + 5, this.xAxisPosY - ((j + 1) * this.scaleHop));
      } else {
        this.context.lineTo(this.yAxisPosX - 0.5, this.xAxisPosY - ((j + 1) * this.scaleHop));
      }

      this.context.stroke();

      if (getConfiguration('scaleShowLabels')) {
        this.context.fillText(calculatedScale['labels'][j], this.yAxisPosX - 8, this.xAxisPosY - (j * this.scaleHop));
      }
    }
    
  }
  
  //@override
  void drawChart(double animPc){
    
    drawLines(animPc);
    
  }

}

==> chart-0.2.4.tar.gz: lib/src/line.dart <==
part of chart;

/**
 * Code based on charts.js by Nick Downie, http://chartjs.org/
 * 
 * (Partial) Dart implementation done by Symcon GmbH, Tim Rasim
 * 
 */

class Line extends DartChart {
  
  int maxSize;
  int scaleHop;
  Map calculatedScale;
  int labelHeight;
  int scaleHeight;
  Map<String, Object> valueBounds;
  int labelHop; //x-distance between two x-axis labels
  int widestXLabel;
  double xAxisLength;
  double yAxisPosX;
  double xAxisPosY;
  int rotateLabels = 0;


  Line(data, config):super(data, config) {

    defaultConfiguration({
        'scaleOverlay' : false, 
        'scaleOverride' : false, 
        'scaleSteps' : null, 
        'scaleStepWidth' : null, 
        'scaleStartValue' : null, 
        'scaleLineColor' : "rgba(0,0,0,.1)", 
        'scaleLineWidth' : 1, 
        'scaleShowLabels' : true, 
        'scaleLabel' : new Map<double, String>(), 
        'scaleFontFamily' : "'Verdana'", 
        'scaleFontSize' : 12, 
        'scaleFontStyle' : "normal", 
        'scaleFontColor' : "#666", 
        'scaleShowGridLines' : true, 
        'scaleGridLineColor' : "rgba(0,0,0,.05)", 
        'scaleGridLineWidth' : 1, 
        'bezierCurve' : true, 
        'pointDot' : true, 
        'pointDotRadius' : 4.0, 
        'pointDotStrokeWidth' : 2, 
        'datasetStroke' : true, 
        'datasetStrokeWidth' : 2, 
        'datasetFill' : true, 
        'animation' : true, 
        'animationSteps' : 60, 
        'animationEasing' : "easeOutQuart", 
        'onAnimationComplete' : null
    });
    
  }
  
  void show(Element parentNode){
    
    super.show(parentNode);

    Map<String, int> drawingSizes = calculateDrawingSizes(this.maxSize, this.rotateLabels);
    this.labelHeight = drawingSizes['labelHeight'];
    this.scaleHeight = drawingSizes['scaleHeight'];
    this.widestXLabel = drawingSizes['widestXLabel']; 
    
    valueBounds = getValueBounds(this.scaleHeight, this.labelHeight);
    if (!getConfiguration('scaleOverride')) {
      this.calculatedScale = calculateScale(this.scaleHeight, this.valueBounds['maxSteps'], this.valueBounds['minSteps'], this.valueBounds['maxValue'], this.valueBounds['minValue']);
    } else {
      this.calculatedScale = {
          'steps' : getConfiguration('scaleSteps'), 
          'stepValue' : getConfiguration('scaleStepWidth'), 
          'graphMin' : getConfiguration('scaleStartValue'), 
          'labels' : []
      };
      populateLabels(this.calculatedScale['labels'], this.calculatedScale['steps'], getConfiguration('scaleStartValue'), getConfiguration('scaleStepWidth'));
    }

    this.scaleHop = (this.scaleHeight / (this.calculatedScale['steps'])).floor();
    calculateXAxisSize();
    
    animationLoop();    
    
  }

  calculateXAxisSize() {
    
    double longestText = 1.0;
    //if we are showing the labels
    if (getConfiguration('scaleShowLabels')) {
      this.context.font = getConfiguration('scaleFontStyle').toString() + " " + getConfiguration('scaleFontSize').toString() + "px " + getConfiguration('scaleFontFamily').toString();
      for (int i = 0; i < this.data['labels'].length; i++) {
        double measuredText = this.context.measureText(this.data['labels'][i].toString()).width;
        longestText = (measuredText > longestText) ? measuredText : longestText;
      }
      
      //also calculate the width of the scales, so they won't be cut off.
      for (int i = 0; i < calculatedScale['labels'].length; i++) {
        double measuredText = this.context.measureText(calculatedScale['labels'][i]).width;
        longestText = (measuredText > longestText) ? measuredText : longestText;
      }
      
      //Add a little extra padding from the y axis
      longestText = longestText + 10;
    }

    this.xAxisLength = this.width.toDouble() - longestText - this.widestXLabel.toDouble();
    
    //this.xAxisLength / (this.data['labels'].length -1) would be correct, but we want one step more
    this.labelHop = (this.xAxisLength / (this.data['labels'].length)).floor();
    
    this.yAxisPosX = this.width.toDouble() - this.widestXLabel / 2 - this.xAxisLength;
    
    this.xAxisPosY = this.scaleHeight.toDouble() + getConfiguration('scaleFontSize')/2;
    
  }
  
  
  /**
   * 
   * draw one step which can consists of two points or multiple inter steps
   * 
   * pointBefore: the last point before this step
   * 
   * returns: the last point inside this step
   */
  Object drawStep(dynamic previousValue, dynamic currentValue, Map dataset, num lastStartX, double currentX, double stepSizeX, double lastStepSizeX, double animPc){  
    
    if (previousValue == null && currentValue == null){
      //if the point before the current point was null, only move the pencil to the point
      this.context.moveTo(xPos(lastStartX), yPos(0.0, animPc));      
      lastStartX = currentX;
      return {
        'currentValue' : null,
        'lastStartX' : lastStartX,
        'lastStepSizeX' : stepSizeX
      };
    }
    
    if (previousValue != null && currentValue == null){
      // the last set of points ends with the last point
      closePath(dataset,lastStartX, currentX -lastStepSizeX);      
      
      return {
        'currentValue' : null,
        'lastStartX' : lastStartX,
        'lastStepSizeX' : stepSizeX
      };
    }
    
    if (previousValue != null && (currentValue is List && currentValue.last == null)){
      //the first value in the list is empty
      
      Object data = {
                     'currentValue' : previousValue,
                     'lastStartX' : lastStartX,
                     'lastStepSizeX' : lastStepSizeX
      };
      stepSizeX = 1 / currentValue.length;
      for (int z = 0; z < currentValue.length; z++)
        data = drawStep(data['currentValue'], currentValue[z],dataset, data['lastStartX'], currentX + stepSizeX * z, stepSizeX, data['lastStepSizeX'], animPc);  

      
      return {
        'currentValue' : data['currentValue'],
        'lastStartX' : data['lastStartX'],
        'lastStepSizeX' : stepSizeX
      };
    }

    if (currentValue != null){
      //continue context from before
      if (previousValue == null){
        lastStartX = currentX;
        if (currentValue is List && currentValue.first is num){
          this.context.moveTo(xPos(lastStartX), yPos(currentValue.first, animPc));
        } else if (currentValue is num){
          this.context.moveTo(xPos(lastStartX), yPos(currentValue, animPc));
        }
      } else if (currentValue is num){
        if (getConfiguration('bezierCurve')) {
          if (previousValue == null){
            previousValue = 0;
          }

          this.context.bezierCurveTo(xPos(currentX  - math.min(stepSizeX, lastStepSizeX)/2), yPos(previousValue.toDouble(), animPc), xPos(currentX  - math.min(stepSizeX, lastStepSizeX)/2), yPos(currentValue.toDouble(), animPc), xPos(currentX), yPos(currentValue, animPc));
        } else {
          this.context.lineTo(xPos(currentX), yPos(currentValue, animPc));
        }
      } 
      
      if (currentValue is List){
        Object data = {
                       'currentValue' : previousValue,
                       'lastStartX' : lastStartX,
                       'lastStepSizeX' : lastStepSizeX
        };
        stepSizeX = 1 / currentValue.length;
        if (previousValue == null){
          data['currentValue'] = currentValue.first;
          data['lastStartX'] = currentX + stepSizeX;
          for (int z = 1; z < currentValue.length; z++){
            data = drawStep(data['currentValue'], currentValue[z],dataset, data['lastStartX'], currentX + stepSizeX * z, stepSizeX, data['lastStepSizeX'], animPc);
          }
        } else {
          for (int z = 0; z < currentValue.length; z++){
            data = drawStep(data['currentValue'], currentValue[z],dataset, data['lastStartX'], currentX + stepSizeX * z, stepSizeX, data['lastStepSizeX'], animPc);  
          }
        }
      lastStartX = data['lastStartX'];
      currentValue = data['currentValue'];
      }
      
      return {
        'currentValue' : currentValue,
        'lastStartX' : lastStartX,
        'lastStepSizeX' : stepSizeX
      };
    }     
    
  }
  
  void drawLines(double animPc) {
    
    //for all: drawStep
    //then draw end if last is not null
    
    this.data['datasets'].forEach( (Map dataset){
      this.context.strokeStyle = dataset['strokeColor'];
      this.context.lineWidth = getConfiguration('datasetStrokeWidth');
      
      this.context.beginPath();
      
      if (dataset['data'][0] is List && dataset['data'][0][0] != null ){
        this.context.moveTo(this.yAxisPosX, this.xAxisPosY - animPc * (calculateOffset(dataset['data'][0][0].toDouble(), this.calculatedScale, this.scaleHop)));
      } else if (dataset['data'][0] is num){
        this.context.moveTo(this.yAxisPosX, this.xAxisPosY - animPc * (calculateOffset(dataset['data'][0].toDouble(), this.calculatedScale, this.scaleHop)));
      }
      
      List test = new List();
      int x = 0;
      Object data = {
       'currentValue' : null,
       'lastStartX' : 0,
       'lastStepSizeX' : 1.0
      };
      dataset['data'].forEach( (value){
        data = drawStep(data['currentValue'], value, dataset, data['lastStartX'], x.toDouble(), 1.0, data['lastStepSizeX'], animPc);
        x++;
      });
      
      if ((dataset['data'].last is num) || (dataset['data'].last is List && dataset['data'].last.last != null)){ 
        closePath(dataset, data['lastStartX'], dataset['data'].length -1);
      } 
      
      if (getConfiguration('pointDot')) {
        this.context.fillStyle = dataset['pointColor'];
        this.context.strokeStyle = dataset['pointStrokeColor'];
        this.context.lineWidth = getConfiguration('pointDotStrokeWidth');
        for (var k = 0; k < dataset['data'].length; k++) {
          if (dataset['data'][k] is List){
            //multiple intermediary steps   
            double stepSizeX = 1 / dataset['data'][k].length;
            int n = 0;
            dataset['data'][k].forEach( (item) {
              if (item != null && item is num){
                this.context.beginPath();
                this.context.arc(xPos(k + stepSizeX *n++ ), this.xAxisPosY - animPc * (calculateOffset(item.toDouble(), this.calculatedScale, this.scaleHop)), getConfiguration('pointDotRadius'), 0, math.PI * 2, true);
                this.context.fill();
                this.context.stroke();
              }
            });
           } else if (dataset['data'][k] is num) {
            //one step
            this.context.beginPath();
            this.context.arc(xPos(k), this.xAxisPosY - animPc * (calculateOffset(dataset['data'][k].toDouble(), this.calculatedScale, this.scaleHop)), getConfiguration('pointDotRadius'), 0, math.PI * 2, true);
            this.context.fill();
            this.context.stroke();
           }
        }
      }
    });
    
  }
  
  void closePath(Map dataset, num startX, num endX){

    this.context.stroke();
       
    if (getConfiguration('datasetFill')) {
      if (dataset['data'][endX.floor()] is List && dataset['data'][endX.floor()].last != null && endX % 1 == 0){
        //1 - 1 / dataset['data'].last.length means the last step exclusive a next full step (confusing but better than no comment at all)
        drawDatasetFill(dataset, startX, endX + 1 - 1 / dataset['data'][endX.floor()].length);
      } else {
        drawDatasetFill(dataset, startX, endX);
      }
    } else {
      this.context.closePath();
      this.context.beginPath();
    }  

  }
  
  /*
   * Draws the background between the points startX and endX
   */
  void drawDatasetFill(Map dataset, num startX, num endX){
      
      this.context.strokeStyle = dataset['fillColor'];
      this.context.lineTo(this.yAxisPosX + (this.labelHop * endX), this.xAxisPosY);
      this.context.stroke();
    
      this.context.strokeStyle = "rgba(0,0,0,0)"; //transparent
      this.context.lineTo(this.yAxisPosX + (this.labelHop * startX), this.xAxisPosY);
      this.context.stroke();
    
      this.context.closePath();
      this.context.fillStyle = dataset['fillColor'];
      this.context.fill();
      
      this.context.beginPath();
      this.context.strokeStyle = dataset['strokeColor'];

  }

  double yPos(num value, double animPc) {
    
    return this.xAxisPosY - animPc * (calculateOffset(value.toDouble(), this.calculatedScale, this.scaleHop));
  
  }

  //FIXME: iterations name is confusing. (suggests it is of the type int, but istn't)
  double xPos(num iteration) {
    
    return this.yAxisPosX + (this.labelHop * iteration);
    
  }

  //@override
  void drawScale() {
    
    //X axis line
    this.context.lineWidth = getConfiguration('scaleLineWidth');
    this.context.strokeStyle = getConfiguration('scaleLineColor');
    this.context.beginPath();
    this.context.moveTo(this.width - this.widestXLabel / 2 + 5, this.xAxisPosY);
    this.context.lineTo(this.width - (this.widestXLabel / 2) - this.xAxisLength - 5, this.xAxisPosY);
    this.context.stroke();


    if (this.rotateLabels > 0) {
      this.context.save();
      this.context.textAlign = 'right';
    } else {
      this.context.textAlign = 'center';
    }
    this.context.fillStyle = getConfiguration('scaleFontColor');
    for (var i = 0; i < this.data['labels'].length; i++) {
      this.context.save();
      if (this.rotateLabels > 0) {
        this.context.translate(this.yAxisPosX + i * this.labelHop, this.xAxisPosY + getConfiguration('scaleFontSize'));
        this.context.rotate(-(this.rotateLabels * (math.PI / 180)));
        this.context.fillText(this.data['labels'][i].toString(), 0, 0);
        this.context.restore();
      } else {
        this.context.fillText(this.data['labels'][i].toString(), this.yAxisPosX + i * this.labelHop, this.xAxisPosY + getConfiguration('scaleFontSize') + 3);
      }

      this.context.beginPath();
      this.context.moveTo(this.yAxisPosX + i * this.labelHop, this.xAxisPosY + 3);

      //Check i isnt 0, so we dont go over the Y axis twice.
      if (getConfiguration('scaleShowGridLines') && i > 0) {
        this.context.lineWidth = getConfiguration('scaleGridLineWidth');
        this.context.strokeStyle = getConfiguration('scaleGridLineColor');
        this.context.lineTo(this.yAxisPosX + i * this.labelHop, 5);
      } else {
        this.context.lineTo(this.yAxisPosX + i * this.labelHop, this.xAxisPosY + 3);
      }
      this.context.stroke();
    }

    //Y axis
    this.context.lineWidth = getConfiguration('scaleLineWidth');
    this.context.strokeStyle = getConfiguration('scaleLineColor');
    this.context.beginPath();
    this.context.moveTo(this.yAxisPosX, this.xAxisPosY + 5);
    this.context.lineTo(this.yAxisPosX, 5);
    this.context.stroke();

    this.context.textAlign = "right";
    this.context.textBaseline = "middle";
    for (int j = 0; j <= calculatedScale['steps']; j++) {
      this.context.beginPath();
      this.context.moveTo(this.yAxisPosX - 3, this.xAxisPosY - ((j + 1) * this.scaleHop));
      if (getConfiguration('scaleShowGridLines')) {
        this.context.lineWidth = getConfiguration('scaleGridLineWidth');
        this.context.strokeStyle = getConfiguration('scaleGridLineColor');
        this.context.lineTo(this.yAxisPosX + this.xAxisLength + 5, this.xAxisPosY - ((j + 1) * this.scaleHop));
      } else {
        this.context.lineTo(this.yAxisPosX - 0.5, this.xAxisPosY - ((j + 1) * this.scaleHop));
      }

      this.context.stroke();

      if (getConfiguration('scaleShowLabels')) {
        this.context.fillText(calculatedScale['labels'][j], this.yAxisPosX - 8, this.xAxisPosY - (j * this.scaleHop));
      }
    }
    
  }
  
  //@override
  void drawChart(double animPc){
    
    drawLines(animPc);
    
  }

}
